## -*-Tcl-*-
 # ###################################################################
 # 
 #	FILE: "genTclapi.tcl"
 #									  created: 19/7/96 {3:40:37 pm}	
 #								  last update: 03/24/1998 {13:09:55 PM}
 #	Author:	Vince Darley
 #	E-mail:	<vince@das.harvard.edu>
 #	  mail:	Division of	Applied	Sciences, Harvard University
 #			Oxford Street, Cambridge MA	02138, USA
 #	   www:	<http://www.fas.harvard.edu/~darley/>
 #	
 # 
 #  modified by  rev reason
 #  -------- --- --- -----------
 #  23/7/96  VMD 1.0 original
 # ###################################################################
 ##

set pl_readfrom "plapi.tcl"
set pl_generating "C++"
set pl_entire_method 0
set pl_small 1

# source this file into tclsh and then use 
# the command 'generatePlapi pltcl_parser.cc'
# to generate everything.

proc generatePlapi {fname} {
    global fout pl_readfrom pl_generating pl_entire_method pl_after
    set fout [open $fname w]
    switch $pl_generating {
	"C++" {
	    if $pl_entire_method {
		puts $fout "int pltcl_stream::parse_tcl_command(tcl_args& arg)\{"
		puts -nonewline $fout "\t"
	    }
	    uplevel \#0 source $pl_readfrom
	    if $pl_entire_method {
		puts $fout "\t\t return tcl_object::parse_tcl_command(arg);"
		puts $fout "\t\}"
		puts $fout "\}"
		puts $fout ""
	    }
	}
	"default" {
	    uplevel \#0 source $pl_readfrom
	}	
    }
    if {[info exists pl_after]} {
	puts $fout $pl_after
    }
    flush $fout
    close $fout
    unset fout
}

proc pltclcmd {args} {
    global fout pl_generating pl_small pl_after
    set help [eval lindex $args 0]
    regsub -all {"} $help {\"} help
    set nameret [eval lindex $args 1]
    set args [eval lrange $args 2 end]
    set name [lindex $nameret 0]
    # Assumes the return type is a single 'word'.
    # This is true for the moment (July 1996)
    set ret [lindex $nameret 1]
    switch $pl_generating {
	"C++" {
	    set syntax ""
	    set reading_values 0
	    if { [string first "&" $args] != -1 } {
		set reading_values 1
	    }
	    foreach arg $args {
		if { [string first "&" $arg] == -1 } {
		    if { [string first ":" $arg] != -1 } {
			append syntax "?[lindex $arg 0]? "
		    } else {
			append syntax "[lindex $arg 0] "
		    }
		}
	    }
	    if { $name != "plflush" } {
		set name [string range [string trim $name] 2 end]
	    }
	    
	    set generated "if\(arg\(\"[string trim ${syntax}]\",\n"
	    append generated "\t\t          \"${help}\"\)\n"
	    append generated "\t\t      ==\"${name}\") \{\n"
	    append generated [generateArgsRead "\t\t" $args]
	    append generated "\t\t${name}\("
	    set argsnum [llength $args]
	    # the nice things about C++ is we don't care if we read or write here
	    foreach arg $args {
		incr argsnum -1
		append generated "[lindex $arg 0]"
		if { $argsnum > 0 } { 
		    append generated $fout ", "
		} else {
		    append generated "\);\n"
		}
	    }
	    if { ![llength $args]} {
		append generated "\);\n"
	    }
	    
	    append generated "\t\tplflush();\n"
	    if { $reading_values } {
		append generated "\t\ttcl_"
		foreach arg $args {
		    if { [string first "&" $arg] != -1 } {
			append generated " << [lindex $arg 0] << lappend"
		    }
		}
		append generated " << result;\n"
	    }
	    append generated "\t\treturn TCL_OK;\n"
	    append generated "\t\} else "
	    puts -nonewline $fout $generated
	}
	"C" {
	    # C version currently defunct
	    puts $fout "/* Autogenerated '$name' */"
	    puts $fout "int tcl_${name}(tcl_args& arg)\{"
	    generateArgsRead "\t" $args
	    puts -nonewline $fout "\t${name}\("
	    set argsnum [llength $args]
	    foreach arg $args {
		incr argsnum -1
		puts -nonewline $fout "[lindex $arg 0]"
		if { $argsnum > 0 } { 
		    puts -nonewline $fout ", "
		} else {
		    puts $fout "\);"
		}
	    }
	    puts $fout "\tplflush();"
	    puts $fout "\treturn TCL_OK;"
	    puts $fout "\}"
	    puts $fout ""
	}
    }
    
}

proc generateArgsRead {indent argnames} {
    foreach arg $argnames {
	set vname [lindex $arg 0]
	set pargs($vname) [lrange $arg 1 end]
	set pos [string first "Def:" $pargs($vname)]
	if { $pos != -1 } {
	    set type [string range $pargs($vname) 0 [expr $pos -1]]
	    set def [string range $pargs($vname) [expr $pos +4] end]
	    append result "${indent}$type $vname = $def;\n"
	} else {			
	    append result "${indent}[string trim $pargs($vname) &] $vname;\n"
	}
	
    }
    set any 0
    foreach arg $argnames {
	if { [string first "&" $arg] == -1 } {
	    set any 1
	}
    }
    if { $any } {	
	if { [llength $argnames] } {
	    append result "${indent}arg"
	    foreach arg $argnames {
		if { [string first "&" $arg] == -1 } {
		    if { [string first "Def:" $arg] != -1 } {
			append result " >> optional"
		    }			
		    append result " >> [lindex $arg 0]"
		}
	    }
	    append result ";" "\n"
	}
    }
    append result "${indent}DONE(arg);" "\n"
    return $result
    
}






